import QtQuick 2.14
import org.kde.kirigami 2.12 as Kirigami
import QtQuick.Controls 2.14 as Controls
import QtQuick.Layouts 1.14 as Layouts
import QtQuick.Controls 1.4 as Old
import QtQuick.Dialogs 1.3 as Dialogs

Kirigami.ApplicationWindow {
    id: root

    title: "Vehicle Voyage"
    
    ListModel {
        id: serviceTypeModel
        ListElement {value: 0; text: qsTr("Oil and Filter Change");}
        ListElement {value: 1; text: qsTr("Windshield Wiper Replacement");}
        ListElement {value: 2; text: qsTr("Coolant Change");}
        ListElement {value: 3; text: qsTr("Tire Rotation");}
        ListElement {value: 4; text: qsTr("Timing Belt Change");}
        ListElement {value: 5; text: qsTr("Serpentine Belt Changed");}
        ListElement {value: -1; text: qsTr("Other");}
    }
    
     contextDrawer: Kirigami.ContextDrawer {
        id: contextDrawer
    }

    Kirigami.PageRouter {
        id: router
        initialRoute: "main"
        pageStack: root.pageStack.columnView
        
        Kirigami.PageRoute {
            name: "main"
            Component {
                Kirigami.Page {
                    id: mainPage
                    mainAction: Kirigami.Action {
                        text: qsTr("Add Vehicle")
                        iconName: "list-add"
                        onTriggered: {
                            router.navigateToRoute(["main","addvehicle"]);
                        }
                    }
                    contextualActions: [
                        Kirigami.Action {
                            iconName: "application-javascript"
                            text: qsTr("Import From Carfax JSON")
                            onTriggered: {
                                importDialog.visible = true;
                            }
                        },
                        Kirigami.Action {
                            iconName: "help-about"
                            text: qsTr("About")
                            onTriggered: {
                                router.navigateToRoute("about");
                            }
                        }
                    ]
                    title: "Vehicles"
                    Dialogs.FileDialog {
                        id: importDialog
                        title: qsTr("Select Carfax Json file to import...")
                        nameFilters: ["Carfax Vehicle Json (*.json)"]
                        onAccepted: {
                            if(jsonConverter.importCarfaxJsonToDB(importDialog.fileUrl)) {
                                //
                            } else {
                                root.showPassiveNotification(qsTr("Failed to parse file."),"short");
                            }
                        }
                    }  
                    Kirigami.CardsListView {
                            anchors.fill: parent
                            id: vehicleView
                            model: vehicleModel
                            delegate: Kirigami.Card {
                                id: card
                                banner {
                                    title: name
                                }
                                header: Row{
                                    topPadding: 10.0
                                    rightPadding: 10.0
                                    leftPadding: 10.0
                                    layoutDirection: Qt.RightToLeft
                                    spacing: 5
                                        Controls.Label {
                                            text: vin
                                            visible: root.wideScreen && mainPage.width > 550
                                            anchors.verticalCenter: parent.verticalCenter
                                        }
                                        Image {
                                            id: vinIcon
                                            source: "qrc:/license.svg"
                                            sourceSize.width: 32
                                            sourceSize.height: 32
                                            anchors.verticalCenter: parent.verticalCenter
                                            MouseArea {
                                                id: vinMouse
                                                hoverEnabled: true
                                                anchors.fill: vinIcon
                                            }
                                            Controls.ToolTip {
                                                id: vinTip
                                                text: vin
                                                visible: vinMouse.containsMouse
                                            }
                                        }
                                }
                                contentItem: Item{
                                    implicitHeight: Kirigami.Units.gridUnit * 4
                                    Layouts.ColumnLayout {
                                        Kirigami.Heading {
                                            text: maker + " " + vmodel
                                            level: 2
                                        }
                                        Kirigami.Heading {
                                            text: year
                                            level: 4
                                        }
                                    }
                                }
                                actions: [
                                    Kirigami.Action {
                                        text: qsTr("Examine Logs")
                                        iconName: "edit-find"
                                        onTriggered: {
                                            router.navigateToRoute(["main", {"route": "servicerecords", "data": {"id": id, "name": name}}]);
                                        }
                                    }
                                ]
                                hiddenActions: [
                                    Kirigami.Action {
                                        text: qsTr("Delete")
                                        iconName: "edit-delete"
                                        onTriggered: {
                                            vehicleModel.removeVehicle(index);
                                        }
                                    }
                                ]
                            }
                            Controls.ScrollBar.vertical: Controls.ScrollBar {}
                    }
                }
            }
        }
    
        Kirigami.PageRoute {
            name: "about"
            Component {
                Kirigami.AboutPage {
                    id: aboutPage
                    actions.main: Kirigami.Action {
                        iconName: "window-close"
                        text: qsTr("Close")
                        onTriggered: {
                            router.navigateToRoute("main");
                        }
                    }
                    aboutData: appAboutData
                }
            }
        }
        
        Kirigami.PageRoute {
            name: "addvehicle"
            Component {
                Kirigami.Page {
                    id: vehicleAddPage
                    title: qsTr("Add Vehicle")
                    Layouts.ColumnLayout {
                        anchors.fill: parent
                        Kirigami.InlineMessage {
                            Layouts.Layout.fillWidth: true
                            Layouts.Layout.leftMargin: 10
                            Layouts.Layout.rightMargin: 10
                            z: 9997
                            type: Kirigami.MessageType.Error
                            showCloseButton: true
                            id: formError
                        }
                        Kirigami.FormLayout {
                            Layouts.Layout.alignment: Qt.AlignHCenter
                            Layouts.Layout.fillWidth: true
                            width: vehicleAddPage.width
                            
                            Item {
                                Kirigami.FormData.isSection: true
                                Kirigami.FormData.label: "Vehicle Information"
                            }
                            
                            Controls.TextField {
                                id: nameField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Vehicle Name")+":"
                            }
                            
                            Controls.TextField {
                                id: makeField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Make")+":"
                            }
                            
                            Controls.TextField {
                                id: modelField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Model")+":"
                            }
                            
                            Controls.TextField {
                                id: yearField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Year")+":"
                                inputMethodHints: Qt.ImhDigitsOnly
                                validator: IntValidator {
                                    bottom: 1900
                                    top: 4000
                                }
                            }
                            
                            Controls.TextField {
                                id: vinField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("VIN")+":"
                            }
                            
                            Layouts.RowLayout {
                                
                                Controls.Button {
                                    text: qsTr("Add")
                                    highlighted: true
                                    onClicked: {
                                        if(yearField.acceptableInput) {
                                            vehicleModel.addNewVehicle(nameField.text, makeField.text, modelField.text, parseInt(yearField.text), "", vinField.text);
                                            router.popRoute();
                                        } else {
                                            formError.text = qsTr("Year is invalid!");
                                            formError.visible = true;
                                        }
                                    }
                                }
                                
                                Controls.Button {
                                    text: qsTr("Cancel")
                                    onClicked: {
                                        router.popRoute();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        
        Kirigami.PageRoute {
            name: "servicerecords"
            Component {
                Kirigami.ScrollablePage {
                    property var serviceId: Kirigami.PageRouter.data.id
                    Component.onCompleted: {
                        recordModel.changeVehicleFilter(serviceId);
                    }
                    id: serviceRecordPage
                    title: qsTr("Service Records for ") + Kirigami.PageRouter.data.name
                    mainAction: Kirigami.Action {
                        text: qsTr("Add Service Record")
                        iconName: "list-add"
                        onTriggered: {
                            router.pushRoute({"route": "addrecord", "data": serviceId});
                        }
                    }
                    /*Layouts.ColumnLayout {
                        anchors.fill: parent
                        Kirigami.InlineMessage {
                            Layouts.Layout.fillWidth: true
                            Layouts.Layout.leftMargin: 10
                            Layouts.Layout.rightMargin: 10
                            z: 9997
                            type: Kirigami.MessageType.Information
                            id: oilMessage
                            showCloseButton: false
                            Component.onCompleted: {
                                //TODO: check if oil change is needed.
                            }
                        }*/
                        ListView {
                                id: serviceView
                                model: recordModel
                                delegate: Kirigami.SwipeListItem {
                                        contentItem: Layouts.ColumnLayout {
                                            Kirigami.Heading {
                                                text: serviceprovider
                                                level: 2
                                            }
                                            Kirigami.Heading {
                                                text: servicetypename
                                                level: 2
                                            }
                                            Controls.Label {
                                                text: new Date(servicedate).toLocaleDateString(Qt.locale())
                                            }
                                            Row {
                                                Image {
                                                    source: "qrc:/speed.svg"
                                                    sourceSize.width: 16
                                                    sourceSize.height: 16
                                                }
                                                Controls.Label {
                                                    text: miles
                                                }
                                            }
                                            Controls.Label {
                                                text: notes
                                            }
                                        }
                                        actions: [
                                            Kirigami.Action {
                                                text: qsTr("Remove")
                                                iconName: "edit-delete"
                                                onTriggered: {
                                                    recordModel.removeRecord(index);
                                                }
                                            }
                                        ]
                                    }
                        }
                    //}
                }
            }
        }
        
        Kirigami.PageRoute {
            name: "addrecord"
            Component {
                Kirigami.Page {
                    id: recordAddPage
                    title: qsTr("Add Service Record")
                    Layouts.ColumnLayout {
                        anchors.fill: parent
                        Kirigami.InlineMessage {
                            Layouts.Layout.fillWidth: true
                            Layouts.Layout.leftMargin: 10
                            Layouts.Layout.rightMargin: 10
                            z: 9997
                            type: Kirigami.MessageType.Error
                            showCloseButton: true
                            id: formError
                        }
                        Kirigami.FormLayout {
                            Layouts.Layout.alignment: Qt.AlignHCenter
                            Layouts.Layout.fillWidth: true
                            width: recordAddPage.width
                            
                            Item {
                                Kirigami.FormData.isSection: true
                                Kirigami.FormData.label: "Service Information"
                            }
                            
                            Controls.TextField {
                                id: providerField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Service Provider")+":"
                            }
                            
                            Controls.ComboBox {
                                id: typeField
                                model: serviceTypeModel
                                textRole: "text"
                                valueRole: "value"
                                onActivated: {
                                    if(currentValue == -1) {
                                        customTypeField.visible = true;
                                    } else {
                                        customTypeField.visible = false;
                                    }
                                }
                                Kirigami.FormData.label: qsTr("Service Type")+":"
                            }
                            
                            Controls.TextField {
                                id: customTypeField
                                visible: false
                                Kirigami.FormData.label: qsTr("Custom Service Type")+":"
                            }
                            
                            Layouts.RowLayout {
                                Kirigami.FormData.label: qsTr("Date")+":"
                                Controls.TextField {
                                    id: dateField
                                    text: new Date().toLocaleDateString(Qt.locale())
                                    readOnly: true
                                }
                                Controls.Button {
                                    icon.name: "office-calendar"
                                    text: qsTr("Select Date")
                                    display: Controls.AbstractButton.IconOnly
                                    onClicked: {
                                        calendarPopup.open();
                                    }
                                }
                            }
                            
                            Controls.TextField {
                                id: milesField
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Miles")+":"
                                inputMethodHints: Qt.ImhDigitsOnly
                                validator: IntValidator {
                                    bottom: 0
                                    top: 500000
                                }
                            }
                            
                            Controls.TextArea {
                                id: notesField
                                placeholderText: qsTr("Notes about service")
                                selectByMouse: true
                                Kirigami.FormData.label: qsTr("Notes")+":"
                            }
                            
                            Layouts.RowLayout {
                            
                                Controls.Button {
                                    text: qsTr("Add")
                                    highlighted: true
                                    onClicked: {
                                        var dateObj = Date.fromLocaleDateString(Qt.locale(), dateField.text);
                                        var typeText = typeField.currentText;
                                        if(typeField.currentValue == -1) {
                                            typeText = customTypeField.text;
                                        }
                                        if(milesField.acceptableInput) {
                                            recordModel.addNewRecord(providerField.text, typeField.currentValue, typeText, dateObj.toISOString(), parseInt(milesField.text), notesField.text);
                                            //TODO: check if emit signal to car warnings.
                                            router.popRoute();
                                        } else {
                                            formError.text = qsTr("Invalid number of miles.");
                                            formError.visible = true;
                                        }
                                    }
                                }
                                Controls.Button {
                                    text: qsTr("Cancel")
                                    onClicked: {
                                        router.popRoute();
                                    }
                                }
                            }
                        }
                    }
                    
                    Kirigami.OverlaySheet {
                        id: calendarPopup
                        Old.Calendar {
                            selectedDate: Date.fromLocaleDateString(Qt.locale(), dateField.text)
                            onClicked: {
                                dateField.text = date.toLocaleDateString(Qt.locale());
                            }
                        }
                    }
                }
            }
        }
    }
}
